require 'helper'
require 'tmpdir'
require 'fileutils'

class TestMicroSqlite3 < Test::Unit::TestCase
  attr :tmpdir
  
  def setup
    @tmpdir = Dir.mktmpdir
  end

  def teardown
    FileUtils.rm_rf(@tmpdir) if @tmpdir
  end

  def db
    @db ||= MicroSql.create "#{tmpdir}/test.sqlite3"
  end
  
  def test_assert_creation_of_db_file_from_url
    path = "#{tmpdir}/db.sqlite3"
    assert !File.exists?(path)

    url = "sqlite://#{path}"
    db = MicroSql.create url
    assert File.exists?(path)
  end
  
  def test_assert_creation_of_db_file_from_path
    path = "#{tmpdir}/db.sqlite3"
    assert !File.exists?(path)

    db = MicroSql.create path
    assert File.exists?(path)
  end
  
  def test_assert_create_tables
    db.execute_batch <<-SQL
    CREATE TABLE t1(idx INTEGER PRIMARY KEY, b INTEGER);
    CREATE TABLE t2(a INTEGER, b INTEGER);
    SQL

    assert_equal %w(t1 t2), db.tables.sort
  end
  
  def test_ask_and_exec
    db.ask "CREATE TABLE t1(idx INTEGER PRIMARY KEY, b INTEGER)"

    # -- insert
    assert_equal 12, db.ask("INSERT INTO t1 (idx,b) VALUES(12, '')")
    assert_equal 13, db.exec("INSERT INTO t1 (idx,b) VALUES(13, '')")

    assert_raise(MicroSql::Error) {  
      assert_equal 1, db.exec("INSERT INTO t1 (idx,b) VALUES(13, '')")
    }
    assert_raise(MicroSql::Error) {  
      assert_equal 1, db.ask("INSERT INTO t1 (idx,b) VALUES(13, '')")
    }
    
    # -- update

    assert_equal 1, db.ask("UPDATE t1 SET b = 'b' WHERE idx=12")
    assert_equal 2, db.ask("UPDATE t1 SET b = 'c'")
    assert_equal 0, db.ask("UPDATE t1 SET b = 'b' WHERE idx=12345")
    
    # assert_equal 1, db.exec("INSERT INTO t1 (idx,b) VALUES(13, '')")
    # assert_equal(13, r)
    # 

    # -- delete
    assert_equal 1, db.ask("DELETE FROM t1 WHERE idx=12")
    assert_equal 0, db.exec("DELETE FROM t1 WHERE idx=12")
    assert_equal 1, db.exec("DELETE FROM t1 WHERE idx=13")
  end

  def test_preparation
    assert_equal(0, db.send(:prepared_queries).length)

    db.exec("SELECT 1")
    assert_equal(1, db.send(:prepared_queries).length)

    db.ask("SELECT 2")
    assert_equal(2, db.send(:prepared_queries).length)

    db.ask("SELECT 2")
    assert_equal(2, db.send(:prepared_queries).length)

    db.exec!("SELECT 3")
    assert_equal(2, db.send(:prepared_queries).length)
  end

  def test_type_conversion
    assert_equal(1, db.ask("SELECT 1"))
    assert_equal('1', db.ask("SELECT '1'"))
  end

  def test_row_conversion
    assert_equal(1, db.ask("SELECT 1"))
    assert_equal([1, 2], db.ask("SELECT 1, 2"))
  end

  def test_arguments
    db.ask "CREATE TABLE t1(idx INTEGER PRIMARY KEY, b INTEGER)"
    db.ask("INSERT INTO t1 (idx,b) VALUES(12, 'twelve')")
    db.ask("INSERT INTO t1 (idx,b) VALUES(13, 'thirteen')")
    db.ask("INSERT INTO t1 (idx,b) VALUES(0, NULL)")
  
    assert_equal 3, db.ask("SELECT COUNT(*) FROM t1")
    assert_equal 12, db.ask("SELECT idx FROM t1 WHERE idx=?", 12)
    assert_equal 12, db.ask("SELECT idx FROM t1 WHERE b=?", "twelve")
    assert_equal 12, db.ask("SELECT idx FROM t1 WHERE b=? AND idx=?", "twelve", 12)
    assert_equal 0, db.ask("SELECT idx FROM t1 WHERE b IS ?", nil)
    assert_equal nil, db.ask("SELECT idx FROM t1 WHERE b=?", nil) # b=NULL is never true but always NULL
  end

  def test_transaction
    db.ask "CREATE TABLE t1(idx INTEGER PRIMARY KEY, b INTEGER)"
    
    count = db.transaction do
      db.ask("INSERT INTO t1 (idx,b) VALUES(12, 'twelve')")
      db.ask("SELECT COUNT(*) FROM t1")
    end
    assert_equal 1, count
     
    assert_equal 1, db.ask("SELECT COUNT(*) FROM t1")

    db.transaction do
      db.ask("INSERT INTO t1 (idx,b) VALUES(13, 'twelve')")
      assert_equal 2, db.ask("SELECT COUNT(*) FROM t1")
      db.rollback!
    end
    
    assert_equal 1, db.ask("SELECT COUNT(*) FROM t1")
  end

  def test_insert
    assert_equal([], db.tables)
    t1 = db.table "CREATE TABLE t1(idx INTEGER PRIMARY KEY, b TEXT)"
    
    assert_equal(["t1"], db.tables)
    assert_equal("t1", t1.table_name)

    assert_equal nil, db.table("t2")

    t1.insert [1, "1"], [2, "2"]
    assert_equal [[1], [2]], db.exec("SELECT idx FROM t1")

    assert_equal(3, t1.insert([3, "3"]))

    assert_equal(4, t1.insert(:idx => 4))
  end

  def test_kv_table
    kv = db.key_value_table("kv")
    assert_equal [ "kv" ], db.tables

    kv["x"] = "y"
    assert_equal 1, db.ask("SELECT COUNT(*) FROM kv")
    assert_equal "y", kv["x"]

    kv["x"] = nil
    assert_equal 0, db.ask("SELECT COUNT(*) FROM kv")
    assert_equal nil, kv["x"]

    count = 0
    
    r = kv.cached "x" do "y" end 
    assert_equal(r, "y")

    r = kv.cached "x" do raise "Yo"; "y" end
    assert_equal(r, "y")
  end
end
